==> 0.1.3.tar.gz: lib/src/model/field_transform.dart <==
part of fields.model;

num _int2num(int value) => value;

int _num2int(num value) => value.toInt();

int _string2int(String value) {
  try {
    return int.parse(value);
  } on FormatException catch(e) {
    throw new FieldTransformError(cause: e);
  }
}

double _string2double(String value) {
  try {
    return double.parse(value);      
  } on FormatException catch(e) {
    throw new FieldTransformError(cause: e);
  }
}

/// Check if a transform from the [sourceType] to the [targetType] can be 
/// performed.  Returns _true_ if such a transform is possible, else _false_.
bool canTransformField(Type sourceType, Type targetType) {
  if (sourceType == targetType) return true;
  return FieldTransform._transforms.any(
      (t) => t.sourceType == sourceType && t.targetType == targetType);
}

/// Transform the given [value] from its [sourceType] to a [targetType].
/// Returns the transformed value.
transformField(value, Type sourceType, Type targetType) {
  if (sourceType == targetType) return value;
  // TODO(rms): check `isAssignableTo` rather than direct equality?
  return FieldTransform._transforms.firstWhere(
      (t) => t.sourceType == sourceType && t.targetType == targetType)(value);
}

/// An error encountered during a transform.
class FieldTransformError extends Error {
  
  /// The caught object that caused the transform to err; may be `null` if none.
  final cause;
  
  /// A description of this transform error.
  final String message;
  
  /// Constructs a new transform error with the optional [cause] and [message].
  FieldTransformError({this.cause, this.message: ''});
  
  String toString() => '$FieldTransformError: $message |> $cause';
}

/// A transform from type [S] to type [T].
class FieldTransform<S, T> {
  
  static final Set<FieldTransform> _transforms = new Set<FieldTransform>();
  
  static bool register(FieldTransform transform) => _transforms.add(transform);
  
  /// Unregister the given [transform].
  /// Returns _true_ if the [transform] was registered, else returns _false_.
  static bool unregister(FieldTransform transform) => 
      _transforms.remove(transform);
  
  static void unregisterAll() => _transforms.clear();
  
  final Function _transform;
  final Function _inverse;
  
  /// The source type of this transform.
  Type get sourceType => S;
  
  /// The target type of this transform.
  Type get targetType => T;
      
  /// Whether or not this transform has an [inverse] function.
  bool get hasInverse => _inverse != null;
  
  /// Creates a new field transform for the given [transform] function and 
  /// optional [inverse] function.
  /// 
  /// If the [inverse] function is not `null` then this transform is a two-way
  /// transform; else it is a one-way transform.
  const FieldTransform(T transform(S), [S inverse(T)]) 
  : _transform = transform
  , _inverse = inverse;
  
  /// Call this transform on [value] and returns the transformed value.
  T call(S value) => _transform(value);
  
  /// Call this transform's inverse on [value] and returns the transformed 
  /// value.
  /// 
  /// If [hasInverse] is false this method will throw an [UnsupportedError].
  S inverse(T value) {
    if (!hasInverse) {
      throw new UnsupportedError('$runtimeType does not have an inverse.');
    }
    return _inverse(value);
  }
}

==> 0.2.3.tar.gz: lib/src/model/field_transform.dart <==
part of fields.model;

num _int2num(int value) => value;

int _num2int(num value) => value.toInt();

int _string2int(String value) {
  try {
    return int.parse(value);
  } on FormatException catch(e) {
    throw new FieldTransformError(cause: e);
  }
}

double _string2double(String value) {
  try {
    return double.parse(value);      
  } on FormatException catch(e) {
    throw new FieldTransformError(cause: e);
  }
}

/// An error encountered during a transform.
class FieldTransformError extends Error {
  
  /// The caught object that caused the transform to err; may be `null` if none.
  final cause;
  
  /// A description of this transform error.
  final String message;
  
  /// Constructs a new transform error with the optional [cause] and [message].
  FieldTransformError({this.cause, this.message: ''});
  
  String toString() => '$FieldTransformError: $message |> $cause';
}

/// A transform from type [S] to type [T].
class FieldTransform<S, T> {
  
  static final Set<FieldTransform> _transforms = new Set<FieldTransform>();
  
  /// Check if a transform from the [sourceType] to the [targetType] can be 
  /// performed.  
  /// 
  /// Returns `true` if such a transform is possible, else `false`.
  static bool canTransform(Type sourceType, Type targetType) {
    if (sourceType == targetType) return true;
    return FieldTransform._transforms.any(
        (t) => t.sourceType == sourceType && t.targetType == targetType);
  }
  
  static bool register(FieldTransform transform) => _transforms.add(transform);
  
  /// Transform the given [value] from its [sourceType] to a [targetType] and
  /// returns the transformed value.
  static transform(value, Type sourceType, Type targetType) {
    if (sourceType == targetType) return value;
    // TODO(rms): check `isAssignableTo` rather than direct equality?
    return _transforms.firstWhere(
        (t) => t.sourceType == sourceType && t.targetType == targetType)(value);
  }
  
  /// Unregister the given [transform].
  /// Returns _true_ if the [transform] was registered, else returns _false_.
  static bool unregister(FieldTransform transform) => 
      _transforms.remove(transform);
  
  static void unregisterAll() => _transforms.clear();
  
  final Function _transform;
  final Function _inverse;
  
  /// The source type of this transform.
  Type get sourceType => S;
  
  /// The target type of this transform.
  Type get targetType => T;
      
  /// Whether or not this transform has an [inverse] function.
  bool get hasInverse => _inverse != null;
  
  /// Creates a new field transform for the given [transform] function and 
  /// optional [inverse] function.
  /// 
  /// If the [inverse] function is not `null` then this transform is a two-way
  /// transform; else it is a one-way transform.
  const FieldTransform(T transform(S), [S inverse(T)]) 
  : _transform = transform
  , _inverse = inverse;
  
  /// Call this transform on [value] and returns the transformed value.
  T call(S value) => _transform(value);
  
  /// Call this transform's inverse on [value] and returns the transformed 
  /// value.
  /// 
  /// If [hasInverse] is false this method will throw an [UnsupportedError].
  S inverse(T value) {
    if (!hasInverse) {
      throw new UnsupportedError('$runtimeType does not have an inverse.');
    }
    return _inverse(value);
  }
}
